@@ -10,7 +10,7 @@ from ocrd_models.ocrd_page import (
     parseString,
     to_xml)
 from ocrd_models.constants import NAMESPACES as NAMESPACES_
-from ocrd_utils import getLogger, xywh_from_points
+from ocrd_utils import getLogger, bbox_from_points
 
 from .utils import (
     set_alto_id_from_page_id,
@@ -18,6 +18,7 @@ from .utils import (
     set_alto_shape_from_coords,
     set_alto_xywh_from_coords,
     setxml,
+    contains,
     get_nth_textequiv)
 from .styles import TextStylesManager, ParagraphStyleManager, LayoutTagManager
 
@@ -43,7 +44,7 @@ REGION_PAGE_TO_ALTO = {
     "Custom": None,
 }
 
-HYPHEN_CHARS = ['-', '⸗', '=']
+HYPHEN_CHARS = ['-', '⸗', '=', '¬', '­']
 
 XSD_ALTO_URLS = {
     '4.2': 'http://www.loc.gov/standards/alto/v4/alto-4-2.xsd',
@@ -67,6 +68,7 @@ class OcrdPageAltoConverter():
         trailing_dash_to_hyp=False,
         textequiv_index=0,
         textequiv_fallback_strategy='last',
+        region_order='document',
         page_filename=None,
         dummy_textline=True,
         dummy_word=True,
@@ -80,9 +82,10 @@ class OcrdPageAltoConverter():
             check_words (boolean): Whether to check if PAGE-XML contains any words before conversion and fail if not
             check_border (boolean): Whether to abort if neither Border nor PrintSpace is defined
             skip_empty_lines (boolean): Whether to omit empty lines completely (True) or create a placeholder empty String in ALTO (False)
-            trailing_dash_to_hyp (boolean): Whether to add a <HYP/> element if the last word in a line ends in ``-``
+            trailing_dash_to_hyp (boolean): Whether to add a <HYP/> element if the last word in a line ends in ``-`` etc
             textequiv_index (int): @index of the TextEquiv to choose
             textequiv_fallback_strategy ("raise"|"first"|"last"): Strategy to handle case of no matchin TextEquiv by textequiv_index
+            region_order ("document"|"reading-order"|"reading-order-only"): The order in which to iterate over regions.
             dummy_textline (boolean): Whether to create a TextLine for regions that have TextEquiv/Unicode but no TextLine
             dummy_word (boolean): Whether to create a Word for TextLine that have TextEquiv/Unicode but no Word
         """
@@ -94,6 +97,7 @@ class OcrdPageAltoConverter():
         self.skip_empty_lines = skip_empty_lines
         self.trailing_dash_to_hyp = trailing_dash_to_hyp
         self.dummy_textline = dummy_textline
+        self.region_order = region_order
         self.dummy_word = dummy_word
         self.logger = logger if logger else getLogger('page-to-alto')
         if pcgts:
@@ -174,7 +178,7 @@ class OcrdPageAltoConverter():
             self.layouttag_mgr.to_xml(self.alto_tags)
 
     def convert_reading_order(self):
-        index_order = [x.id for x in self.page_page.get_AllRegions(order='reading-order', depth=0)]
+        index_order = [x.id for x in self.page_page.get_AllRegions(order='reading-order-only', depth=0)]
         for id_cur, id_next in zip(index_order[:-1], index_order[1:]):
             self.alto_printspace.find('.//*[@ID="%s"]' % id_cur).set('IDNEXT', id_next)
 
@@ -183,58 +187,60 @@ class OcrdPageAltoConverter():
         page_height = self.page_page.imageHeight
         setxml(self.alto_page, 'WIDTH', page_width)
         setxml(self.alto_page, 'HEIGHT',  page_height)
-        page_printspace = self.page_page.get_PrintSpace()
-        dummy_printspace = False
-        if page_printspace is None:
-            self.logger.warning("PAGE-XML has no PrintSpace, trying to fall back to Border")
-            page_printspace = self.page_page.get_Border()
-            if page_printspace is None:
-                dummy_printspace = True
+        page_border = self.page_page.get_Border()
+        page_pspace = self.page_page.get_PrintSpace()
+        if page_pspace is None and page_border is not None:
+            self.logger.warning("PAGE-XML has Border but no PrintSpace - Margins will be empty")
+            page_pspace = page_border
+        elif page_border is None and page_pspace is not None:
+            self.logger.warning("PAGE-XML has PrintSpace but no Border - Margins will be empty")
+            page_border = page_pspace
+        elif page_border is None and page_pspace is None:
+            self.logger.warning("PAGE-XML has neither Border nor PrintSpace - PrintSpace will fill the image")
+            alto_pspace = ET.SubElement(self.alto_page, 'PrintSpace')
+            setxml(alto_pspace, 'VPOS', 0)
+            setxml(alto_pspace, 'HPOS', 0)
+            setxml(alto_pspace, 'HEIGHT', page_height)
+            setxml(alto_pspace, 'WIDTH', page_width)
+            return alto_pspace
 
-        alto_printspace = ET.SubElement(self.alto_page, 'PrintSpace')
-        if dummy_printspace:
-            self.logger.warning("PAGE-XML has neither Border nor PrintSpace")
-            for pos in ('Top', 'Left', 'Right', 'Bottom'):
-                margin = ET.SubElement(self.alto_page, '%sMargin' % pos)
-                for att in ('VPOS', 'HPOS', 'HEIGHT', 'WIDTH'):
-                    setxml(margin, att, 0)
-        else:
-            xywh = xywh_from_points(page_printspace.get_Coords().points)
+        alto_pspace = ET.SubElement(self.alto_page, 'PrintSpace')
+        set_alto_xywh_from_coords(alto_pspace, page_pspace)
+        set_alto_shape_from_coords(alto_pspace, page_pspace)
+
+        if page_border is not page_pspace:
+            bmin_x, bmin_y, bmax_x, bmax_y = bbox_from_points(page_border.get_Coords().points)
+            pmin_x, pmin_y, pmax_x, pmax_y = bbox_from_points(page_pspace.get_Coords().points)
+            # 
+            #  ╔═══════╗   ╔═══════╗   ╔╗   ╔══╗
+            #  ║┌───┐  ║   ╚═══════╝   ║║   ║  ║              ┌───┐
+            #  ║│   │  ║ →           + ║║   ║  ║ (margins) +  │   │ (pspace)
+            #  ║└───┘  ║   ╔═══════╗   ║║   ║  ║              └───┘
+            #  ║       ║   ║       ║   ║║   ║  ║
+            #  ╚═══════╝   ╚═══════╝   ╚╝   ╚══╝
+            #
             alto_topmargin = ET.SubElement(self.alto_page, 'TopMargin')
-            setxml(alto_topmargin, 'VPOS', 0)
-            setxml(alto_topmargin, 'HPOS', 0)
-            setxml(alto_topmargin, 'HEIGHT', xywh['x'])
-            setxml(alto_topmargin, 'WIDTH', page_width)
+            setxml(alto_topmargin, 'VPOS', bmin_y)
+            setxml(alto_topmargin, 'HPOS', bmin_x)
+            setxml(alto_topmargin, 'HEIGHT', pmin_y - bmin_y)
+            setxml(alto_topmargin, 'WIDTH', bmax_x - bmin_x)
             alto_leftmargin = ET.SubElement(self.alto_page, 'LeftMargin')
-            setxml(alto_leftmargin, 'VPOS', 0)
-            setxml(alto_leftmargin, 'HPOS', 0)
-            setxml(alto_leftmargin, 'HEIGHT', page_height)
-            setxml(alto_leftmargin, 'WIDTH', xywh['x'])
+            setxml(alto_leftmargin, 'VPOS', bmin_y)
+            setxml(alto_leftmargin, 'HPOS', bmin_x)
+            setxml(alto_leftmargin, 'HEIGHT', bmax_y - bmin_y)
+            setxml(alto_leftmargin, 'WIDTH', pmin_x - bmin_x)
             alto_rightmargin = ET.SubElement(self.alto_page, 'RightMargin')
-            setxml(alto_rightmargin, 'VPOS', 0)
-            setxml(alto_rightmargin, 'HPOS', xywh['x'] + xywh['w'])
-            setxml(alto_rightmargin, 'HEIGHT', page_height)
-            setxml(alto_rightmargin, 'WIDTH', page_width - (xywh['x'] + xywh['w']))
+            setxml(alto_rightmargin, 'VPOS', bmin_y)
+            setxml(alto_rightmargin, 'HPOS', pmax_x)
+            setxml(alto_rightmargin, 'HEIGHT', bmax_y - bmin_y)
+            setxml(alto_rightmargin, 'WIDTH', bmax_x - pmax_x)
             alto_bottommargin = ET.SubElement(self.alto_page, 'BottomMargin')
-            setxml(alto_bottommargin, 'VPOS', xywh['y'] + xywh['h'])
-            setxml(alto_bottommargin, 'HPOS', 0)
-            setxml(alto_bottommargin, 'HEIGHT', page_height - (xywh['y'] + xywh['h']))
-            setxml(alto_bottommargin, 'WIDTH', page_width)
-            set_alto_xywh_from_coords(alto_printspace, page_printspace)
-            if version.parse(self.alto_version) >= version.parse('3.1'):
-                set_alto_shape_from_coords(alto_printspace, page_printspace)
+            setxml(alto_bottommargin, 'VPOS', pmax_y)
+            setxml(alto_bottommargin, 'HPOS', bmin_x)
+            setxml(alto_bottommargin, 'HEIGHT', bmax_y - pmax_y)
+            setxml(alto_bottommargin, 'WIDTH', bmax_x - bmin_x)
 
-        if dummy_printspace:
-            setxml(alto_printspace, 'VPOS', 0)
-            setxml(alto_printspace, 'HPOS', 0)
-            setxml(alto_printspace, 'HEIGHT', page_height)
-            setxml(alto_printspace, 'WIDTH', page_width)
-        else:
-            set_alto_xywh_from_coords(alto_printspace, page_printspace)
-            if version.parse(self.alto_version) >= version.parse('3.1'):
-                set_alto_shape_from_coords(alto_printspace, page_printspace)
-
-        return alto_printspace
+        return alto_pspace
 
     def convert_metadata(self):
         alto_measurementunit = ET.SubElement(self.alto_description, 'MeasurementUnit')
@@ -331,12 +337,28 @@ class OcrdPageAltoConverter():
                 self._convert_textlines(textblock_alto, parent_page)
 
     def convert_text(self):
-        for reg_page in self.page_page.get_AllRegions(depth=0):
+        for reg_page in self.page_page.get_AllRegions(depth=0, order=self.region_order):
             reg_page_type = reg_page.__class__.__name__[0:-10] # len('RegionType') == 10
             reg_alto_type = REGION_PAGE_TO_ALTO[reg_page_type]
             if not reg_alto_type:
                 raise ValueError("Cannot handle PAGE-XML %sRegion" % reg_page_type)
-            reg_alto = ET.SubElement(self.alto_printspace, reg_alto_type)
+            # determine if the region belongs to PrintSpace or to any of the Margins
+            reg_bbox = bbox_from_points(reg_page.get_Coords().points)
+            if contains(self.alto_printspace, reg_bbox):
+                parent = self.alto_printspace
+            else:
+                parent = None
+                for margin in ['LeftMargin', 'RightMargin', 'TopMargin', 'BottomMargin']:
+                    if not hasattr(self.alto_page, margin):
+                        continue
+                    margin = getattr(self.alto_page, margin)
+                    if contains(margin, reg_bbox):
+                        parent = margin
+                        break # pick first match only
+                if not parent:
+                    parent = self.alto_printspace
+                    self.logger.warning("region '%s' not properly contained in PrintSpace or Margins", reg_page.id)
+            reg_alto = ET.SubElement(parent, reg_alto_type)
             set_alto_id_from_page_id(reg_alto, reg_page)
             set_alto_xywh_from_coords(reg_alto, reg_page)
             if version.parse(self.alto_version) >= version.parse('3.1'):
@@ -351,10 +373,6 @@ class OcrdPageAltoConverter():
                 self._convert_textlines(reg_alto, reg_page)
             elif reg_page_type == 'Table':
                 self._convert_table(reg_alto, reg_page)
-            elif reg_page_type in ('Image', 'Separator'):
-                pass # nothing more to do
-            else:
-                raise ValueError('Unhandled region type %s' % reg_page_type)
 
     def _set_dummy_x_for_y(self, el, parent_level):
         child_level = 'Word' if parent_level == 'TextLine' else 'TextLine'
